=begin
Copyright 2011, Matt666
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   		Free

Author:     	Matthieu NOBLET
Organization:   Matt666
Name:      		Cartouche
Version:   		1.20
SU Version:   	6 til 8
Date:      		13/07/2011
Description:   	Create a text contains title informations
Usage:      	Type "Matt_Tools::Cartouche::go" in the ruby console to create / edit cartouche.
				Type "Matt_Tools::Cartouche::settings" in the ruby console to go to the cartouche settings.
				You can record an existing text as a cartouche. Type "Matt_Tools::Cartouche::getExistingCartouche" in the ruby console.
History:
	1.2			20110712
				Help file added
				%DIR variable (Directory)
				%PAGE variable (add name of the current scene name in the cartouche. And every change of page will also modify cartouche.
				%N variable (add page number in the cartouche)
				_/_ marker to put some texts following others
				Modify call of case change
				Now you must show where the information is in the title. So now you can change order of titel information
				Possibilty to put view tooltip (just iso views) instead of scene tab name (if there is an iso view)
	
	1.101		Little bug
				Language (FR - EN)
	
	1.100		New sort order: Cartouche, and not title

	1.000		20110623
				First version
=end

require 'sketchup.rb'
# require 'langhandler'
#Supports organizer.rb
unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	dir=File.dirname(__FILE__),"Matt_Cartouche"
	strings = {}
	if ["fr","FR","fr-FR"].include?(Sketchup.os_language)
		f=IO.readlines(File.join(dir,"/cartouche.fr"))
	else
		f=IO.readlines(File.join(dir,"/cartouche.en"))
	end
	n=0
	f.each do |l|
		l=l.strip.split("___")
		strings[l.first]=l[1]
	end
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Plugins"))
	cartouche=organizer.add_submenu strings['cartouche']
	cartouche.add_item(strings['add']){ Matt_Tools::Cartouche::go }
	cartouche.add_item(strings['settings']){ Matt_Tools::Cartouche::settings }
	cartouche.add_item(strings['record']){ Matt_Tools::Cartouche::getExistingCartouche }
	cartouche.add_separator
	cartouche.add_item(strings['help']){ UI::openURL File.join(dir, "Matt_Cartouche_Help.html") }
	
	tb = UI::Toolbar.new(strings['cartouche'])
	dir = File.dirname(__FILE__), 'Matt_Cartouche'
	
	txt=strings['add_cartouche']
	cmd = UI::Command.new(txt) { Matt_Tools::Cartouche::go }
		cmd.small_icon = File.join(dir, "cartouche16.png")
		cmd.large_icon = File.join(dir, "cartouche24.png")
		cmd.tooltip = cmd.status_bar_text = txt
		tb.add_item(cmd)
		
	txt=strings['cartouche_settings']
	cmd = UI::Command.new(txt) { Matt_Tools::Cartouche::settings }
		cmd.small_icon = File.join(dir, "settings16.png")
		cmd.large_icon = File.join(dir, "settings24.png")
		cmd.tooltip = cmd.status_bar_text = txt
		tb.add_item(cmd)
end

module Matt_Tools
module Cartouche
	dir=File.dirname(__FILE__),"Matt_Cartouche"
	@strings = {}
	if ["fr","FR","fr-FR"].include?(Sketchup.os_language)
		f=IO.readlines(File.join(dir,"/cartouche.fr"))
	else
		f=IO.readlines(File.join(dir,"/cartouche.en"))
	end
	n=0
	f.each do |l|
		l=l.strip.split("___")
		@strings[l.first]=l[1]
	end
end

def Cartouche.go
	puts 'Matt_Tools::Cartouche::go'
	model=Sketchup.active_model
	i=[]
	sel=model.entities.select{|e|e.class==Sketchup::Text}
	sel.each{|e|i.push(e) if e.get_attribute("Matt_Cartouche",'existing')}
	if model.get_attribute("Matt_Cartouche",'settings')
		settings=model.get_attribute("Matt_Cartouche",'settings')
	else
		settings=model.set_attribute("Matt_Cartouche",'settings',Cartouche::default_settings)
	end
	
	lst=Cartouche::create_text(settings,false)
	return if not lst
	txt=Cartouche::to_s(lst[0])
	
	msg='Cartouche'
	Sketchup.version.to_i>=7 ? model.start_operation(msg,true) : model.start_operation(msg)

	if i.length>0
		i=i.first
		i.text=txt
	else
		while not model.entities.parent==model
			model.close_active
		end
		i=model.add_note(txt,settings[1].to_f,settings[2].to_f)
		i.set_attribute("Matt_Cartouche",'existing',true)
	end
	if lst[1]==true
		if not model.get_attribute("Matt_Cartouche",'pobs')
			model.pages.add_observer(Matt_Cartouche_Pages_Observer.new)
			model.set_attribute("Matt_Cartouche",'pobs',true)
		end
	end
	model.commit_operation
end #go

def Cartouche.create_text(lst,flag)
	model=Sketchup.active_model
	sep=lst[0]
	title=model.title
		if not title.include?(sep)
		UI.messagebox "#{@strings['sepmsg1']}#{sep}#{@strings['sepmsg2']}"
		return
	end
	title=title.split(sep)
	counter=6 #counter of the results
	cn=0 #counter of the cartouche
	str={}
	pages=false
	while counter<lst.length
		line=lst[counter]
		marker=false
		if line!=nil and line!=("")
			value=lst[counter+1]
			if value[-3..-1]=="_/_"
				marker=true
				value=value[0..-4]
			end
			if value[0]==37 #Value is a variable (37="%")
				case value
				when "%DATE" 
					str[cn]="#{line}: #{Time.now.to_a[3..5].join'/'}"
				when "%PAGE"
					pages=true
					if lst[5]=="Yes"
						if tooltip=Cartouche::tooltip
							str[cn]="#{line}: #{tooltip}"
						else
							str[cn]="#{line}: #{model.pages.selected_page.name}"
						end
					else
						str[cn]="#{line}: #{model.pages.selected_page.name}"
					end
				when "%N"
					pages=true
					str[cn]="#{line}: #{1+(model.pages.to_a.index(model.pages.selected_page))}"
				when "%DIR"
					str[cn]="#{line}: #{model.path.split("\\")[0..-2].join("\\")}"
				else
					UI.messagebox "#{@strings['v1']} '#{value}'\n#{@strings['v2']}"
					return nil
				end
			elsif value[0]==64 #Value contained in drawing title (64="@")
				if not value[1].respond_to?("chr")
					UI.messagebox "#{@strings['v4']} #{counter} '#{value}'\n#{@strings['v2']}"
					return nil
				end
				n=(value[1].chr.to_i)-1
				if title[n]
					t_part=title[n]
					if value[3..-1] and value[3]==40 and value[-1]==41 #40="(" and 41=")"
						interval=value[4..-2]
						if interval.include?("..")
							int=interval.split("..")
							t_part=t_part[(int[0].to_i)..(int[1].to_i)]
						elsif interval.include?(44)
							int=interval.split(",")
							puts int
							t_part=t_part[(int[0].to_i),(int[1].to_i)]
						else
							t_part=t_part[interval.to_i].chr
						end
					end
					case value[2]
					when 62 #62=">"
						str[cn]="#{line}: #{t_part.upcase}"
					when 60 #60="<"
						str[cn]="#{line}: #{t_part.capitalize}"
						# t_partn=""
						# t_part.split(" ").each{|p|t_partn="#{t_partn} #{p[0].chr.upcase}#{p[1..-1]}"}
						# str[cn]="#{line}: #{t_partn}"
					when 61,nil #61="="
						str[cn]="#{line}: #{t_part}"
					end
				else
					UI.messagebox "#{@strings['v3']} '#{value}'\n#{@strings['v2']}"
					return nil
				end
			elsif value!=""
				str[cn]="#{line}: #{value}"
			end
			str[cn]=str[cn]+"_/_" if marker
			cn+=1
		end
		counter+=2
	end
	lst=[]
	cn=0
	while cn<str.size
		lst.push str[cn]
		cn+=1
	end
	flag=true if pages
	if pages
		model.set_attribute("Matt_Cartouche",'pages',true)
	else
		model.set_attribute("Matt_Cartouche",'pages',false)
	end
	return [lst,flag]
end

def Cartouche.to_s(lst)
	model=Sketchup.active_model
	settings=model.get_attribute("Matt_Cartouche",'settings')
	csepd=settings[3]
	csepf=settings[4]
	n=0
	while n<lst.length
		temp_t=lst[n]
		if temp_t[-3..-1]=="_/_"
			txt="#{txt}#{csepd}#{temp_t[0..-4]}#{csepf}"
		else
			if txt
				txt="#{txt}\n#{temp_t}"
			else
				txt="#{temp_t}"
			end
		end
		n+=1
	end
	return txt
end

#Settings
def Cartouche.settings
	puts 'Matt_Tools::Cartouche::settings'
	model=Sketchup.active_model
	if model.get_attribute("Matt_Cartouche",'settings')
		settings=model.get_attribute("Matt_Cartouche",'settings')
	else
		settings=Cartouche::default_settings
	end
	prompt = [@strings['tsep'],@strings['x'],@strings['y'],@strings['csep1'],@strings['csep2'],@strings['ttmode'],@strings['1n'],@strings['1v'],@strings['2n'],@strings['2v'],@strings['3n'],@strings['3v'],@strings['4n'],@strings['4v'],@strings['5n'],@strings['5v'],@strings['6n'],@strings['6v'],@strings['7n'],@strings['7v'],@strings['8n'],@strings['8v']]
	list=["","","","","",@strings['yn'],"","","","","","","","","","","","","","","",""]
	default = [settings[0],settings[1],settings[2],settings[3],settings[4],settings[5],settings[6],settings[7],settings[8],settings[9],settings[10],settings[11],settings[12],settings[13],settings[14],settings[15],settings[16],settings[17],settings[18],settings[19],settings[20],settings[21]]
	results = inputbox(prompt,default,list,@strings['label'])
	if results
		model.set_attribute("Matt_Cartouche",'settings',results)
	end
end #settings

# My personal settings
def Cartouche.default_settings
	puts 'Matt_Tools::Cartouche::default_settings'
	return ["#", "0.02", "0.042", " (", ")", "No", "Affaire", "@1>", "Localisation", "@2>", "Designation", "@3<", "Date", "%DATE", "Indice", "@4>(-1)", "", "", "", "", "", ""]
end

# When a cartouche was created before this tool
def Cartouche.getExistingCartouche
	model=Sketchup.active_model
	if model.selection.empty?
		puts 'No selection. Please select a cartouche.'
		UI.messagebox @strings['sel']
	else
		i=[]
		sel=model.entities.select{|e|e.class==Sketchup::Text}
		sel.each{|e|i.push(e) if e.get_attribute("Matt_Cartouche",'existing')}
		if i.length>0
			i.each{|e|e.delete_attribute("Matt_Cartouche",'existing')}
		end
		e=model.selection.first
		if e.class==Sketchup::Text
			if e.has_leader?
				UI.messagebox @strings['text']
			else
				e.set_attribute("Matt_Cartouche",'existing',true)
				puts e
			end
		else 
			UI.messagebox @strings['selection']
		end
	end
end

def Cartouche.tooltip
	l={}
	l["Front"]=[Geom::Vector3d.new(1,0,0),Geom::Vector3d.new(0,0,1),Geom::Vector3d.new(0,1,0)]
	l["Back"]=[Geom::Vector3d.new(-1,0,0),Geom::Vector3d.new(0,0,1),Geom::Vector3d.new(0,-1,0)]
	l["Top"]=[Geom::Vector3d.new(1,0,0),Geom::Vector3d.new(0,1,0),Geom::Vector3d.new(0,0,-1)]
	l["Bottom"]=[Geom::Vector3d.new(-1,0,0),Geom::Vector3d.new(0,1,0),Geom::Vector3d.new(0,0,1)]
	l["Left"]=[Geom::Vector3d.new(0,-1,0),Geom::Vector3d.new(0,0,1),Geom::Vector3d.new(1,0,0)]
	l["Right"]=[Geom::Vector3d.new(0,1,0),Geom::Vector3d.new(0,0,1),Geom::Vector3d.new(-1,0,0)]
	c=Sketchup.active_model.active_view.camera
	axis=[c.xaxis,c.yaxis,c.zaxis]
	for s in l.keys
		return s if l[s]==axis
	end
	return nil
end

end #module Matt666

#Class used if %PAGE or %N variable is called.
class Matt_Cartouche_Pages_Observer < Sketchup::PagesObserver
def onContentsModified(pages)
	model=Sketchup.active_model
	i=[]
	sel=model.entities.select{|e|e.class==Sketchup::Text}
	sel.each{|e|i.push(e) if e.get_attribute("Matt_Cartouche",'existing')}
	return if not i.length>0
	i=i.first
	settings=model.get_attribute("Matt_Cartouche",'settings')
	lst=Matt_Tools::Cartouche::create_text(settings,true)
	txt=Matt_Tools::Cartouche::to_s(lst[0])
	i.text=txt
end
end #Pages class

#Class used if you open an existing drawing
class Matt_Cartouche_Model_Observer < Sketchup::AppObserver
def onOpenModel(model)
	if model.get_attribute("Matt_Cartouche",'pages')
		model.pages.add_observer(Matt_Cartouche_Pages_Observer.new)
		model.set_attribute("Matt_Cartouche",'pobs',true)
	end
end
end#App class

#Attach the app observer
Sketchup.add_observer(Matt_Cartouche_Model_Observer.new)

model=Sketchup.active_model
#Used if you open SU directly with a drawing by double clicking. It attach pages observer if created pages
if model.get_attribute("Matt_Cartouche",'pages') and model.pages
	model.pages.add_observer(Matt_Cartouche_Pages_Observer.new)
	model.set_attribute("Matt_Cartouche",'pobs',true)
end
